#include <stdio.h>
#include <stdlib.h>
#include "Audio.h"

#include <GL/glut.h>
#include <time.h>  
 
/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "GameObj.h"
#include "GameObjTimed.h"
#include "Bullet.h"
#include "Ship.h"
#include "GameSession.h"
#include "GameIntro.h"
#include "utility.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

/** Global Variable storing the current Game Section
It can point either to GS or to GI */
GameSection *Game;

/** Global Variable storing the Game Session */
GameSession GS;

/** Global Variable storing the Game intro*/
GameIntro GI;

/** @name Audio Stuff
@memo All we need for playing music and effects
	*/
//@{
/// Seal Audio Global Variable storing the fire sample
    LPAUDIOWAVE lpWaveFire;
/// Seal Audio Global Variable storing the Explosion sample
		LPAUDIOWAVE lpWaveShipExp;
/// Seal Audio Global Variable storing the Explosion sample
		LPAUDIOWAVE lpWaveExp;
/// Seal Audio Global Variable storing the Background music
	  LPAUDIOMODULE lpModule;
   
/// Seal Audio Global Variable storing the voice for the fire 
		HAC hVoiceFire; 
/// Seal Audio Global Variable storing three voices for the explosion sample
    HAC hVoiceExp[3]; 

/** All the Seal setup.
Loading of modules and effects and opening all the needed voices.
*/
void AudioSetup()
{   
	AUDIOINFO info;
 
  /* initialize audio library */
  AInitialize();

  /* open audio device */
  info.nDeviceId = AUDIO_DEVICE_MAPPER;
  info.wFormat = AUDIO_FORMAT_16BITS | AUDIO_FORMAT_STEREO;
  info.nSampleRate = 44100;
  AOpenAudio(&info);

  /* load module and waveform file */
  ALoadModuleFile("data/intro.xm", &lpModule, 0);
  ALoadWaveFile("data/fire.wav", &lpWaveFire, 0);
	ALoadWaveFile("data/explode.wav", &lpWaveExp, 0);
	ALoadWaveFile("data/shipexplode.wav", &lpWaveShipExp, 0);
  assert(lpModule && lpWaveFire && lpWaveExp);
  /* open voices for module and waveform */
  AOpenVoices(lpModule->nTracks + 4);


  /* play the waveform through a voice */
  ACreateAudioVoice(&hVoiceFire);
  ACreateAudioVoice(&(hVoiceExp[0]));
  ACreateAudioVoice(&(hVoiceExp[1]));
  ACreateAudioVoice(&(hVoiceExp[2]));
    
	assert(hVoiceFire && hVoiceFire); 
}

/** Seal CleanUp.
Stop all the voices, destroy them stop and close the module,
 and free all  the loaded things.
*/
void AudioEnd()
{
  AStopVoice(hVoiceFire);
  AStopVoice(hVoiceExp[0]);
  AStopVoice(hVoiceExp[1]);
  AStopVoice(hVoiceExp[2]);
  ADestroyAudioVoice(hVoiceFire);
  ADestroyAudioVoice(hVoiceExp[0]);
  ADestroyAudioVoice(hVoiceExp[1]);
  ADestroyAudioVoice(hVoiceExp[2]);
    
	/* stop playing the module */
  AStopModule();
  ACloseVoices();

	/* release the waveform & module */
	AFreeWaveFile(lpWaveExp);
	AFreeWaveFile(lpWaveFire);
	AFreeModuleFile(lpModule);

  /* close audio device */
	ACloseAudio();
}
//@}    
/** @name GLUT callbacks
@memo These functions are called back by GLUT main loop
	*/
//@{

/** Glut Callback on standard key input.
Mainly to get the space key shoot and esc quitting.

Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.3s Added pause management
\item 0.4 Passed everything, apart from esc-quitting, to Game->Skey()
\item 0.5 Added audio closing 
\end{itemize}
} */
void Key(unsigned char key, int x, int y)
{
  switch (key) {
  case 27:
		AudioEnd();
    exit(0);
  break;
  }
	Game->Key(key);
}

/** Glut Callback on pression of arrow keys.
It should start the thrust (up arrow), 
the rotation (left/right arrow) and side thrust 
(alt+left/right) of  the ship.
Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.2s Added side movemente management.
\item 0.4 Passed everything to Game->Skey()
\end{itemize}
}
*/
void  SpecialKey(int key, int x, int y)
{
	Game->SKey(key);
}
/** Glut Callback on release of arrow keys.
It should stop the (side and back) thrust and the rotation of the ship.

Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.2s Added side movemente management
\item 0.4 Passed everything to Game->SkeyUp()
\end{itemize}
}
*/
void SpecialKeyUp(int key, int x, int y)
{
	Game->SKeyUp(key);
}

/** Opengl Initialization.
Initialization of standard things for Shaded display:
\begin{itemize}
\item Light Setup (position, enabling and light model)
\item Enable Culling and and disable double face materials. 
Note that you should pay attention to always draw faces 
with correct vertex ordering.
\item Enable Depth test and color_material (to change material with a simple glColor)
\end{itemize}
Version History {\small 
\begin{itemize}
\item 0.1 First Release (almost nothing here.
\item 0.2 Added light setup, depth test, culling disable and double sided lighting
\item 0.3 Enabled Culling and Disabled double sided lighting (Better performance!)	
\item 0.3s Added Explosion light setup (color and quadratic attenuation)
\item 0.4 Deleted!
\end{itemize}}

*/
void myGLInit(void)
{
	
} 

/** Glut Callback on Window Reshape.
It updates the Projection Matrix and the viewport and stores the
new window size into memebers of GameSession object. Updated in V2
to
Version History {\small 
\begin{itemize}
\item 0.1 First Release (almost nothing here.
\item 0.2 correctly handle non square windows and always show the whole 
(square) playfield.
\item 0.4 Passed everything to Game->SkeyUp()
\end{itemize}}
*/
void Reshape(GLsizei w, GLsizei h)
{ 
	Game->Reshape(w,h);
  
}		

/** Glut Idle Callback .
It is continuously called when events are not
being received. It post a refresh request in order to make the animation
as smoother as possible. It is enabled/disable by the Visible Callback
@see Visible
*/

void Idle(void)
{
	glutPostRedisplay();
}
/** Glut Callback on Window visibility.
It is called when the glut window change its visiblitiy status.
When the window is hidden we should stop the idle callback (and viceversa).
*/
void Visible(int vis)
{
 if (vis == GLUT_VISIBLE)
      glutIdleFunc(Idle);
 else
      glutIdleFunc(NULL);
}


/** Glut Callback to Draw everything.
It clear the screen, call the GameSection 
Update() and Draw() and swap the buffers.
It keep also track of the FPS rate.
*/
void Display(void)
{ 
  static int lasttime;
  int dt,nt;
  dt=glutGet(GLUT_ELAPSED_TIME)-lasttime;
  lasttime=glutGet(GLUT_ELAPSED_TIME);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glutGet(GLUT_ELAPSED_TIME);
  Game->Update(dt);
  nt=Game->Draw();
	float fps=FPS(dt);
	glColor3f(1,1,1);
	glPrintf(10,10,14,"fps %7.1f tri %8i tri/sec %8.0f\n",fps,nt,nt*fps);
  
  glutSwapBuffers();
}


/** The Classical Main Function
It has to setup all the glut callbacks, initialize Openglstatus
and start the glutMainLoop.
*/
int main(int argc, char** argv)
{ 
	AudioSetup();
	Game=&GI;
  glutInit(&argc, argv);
  glutInitWindowSize(512, 512);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutCreateWindow("Asteroids");

  glutReshapeFunc(Reshape);
  glutDisplayFunc(Display);
  glutIdleFunc(Idle);
	glutVisibilityFunc(Visible);
  glutKeyboardFunc(Key);	
  glutSpecialFunc(SpecialKey);	
  glutSpecialUpFunc(SpecialKeyUp);	
    
  myGLInit(); 
  srand48(time(NULL));	
	Game->Init();
  glutMainLoop();

  return 0;             
}
//@}
